@@ -27,9 +27,9 @@
 #include <cairo-ft.h>
 #pragma GCC diagnostic pop
 
-#include "pango-fontmap.h"
+#include "pangofc-fontmap-private.h"
 #include "pangocairo-private.h"
-#include "pangocairo-fc.h"
+#include "pangocairo-fc-private.h"
 #include "pangofc-private.h"
 #include "pango-impl-utils.h"
 
@@ -118,7 +118,7 @@ pango_cairo_fc_font_get_glyph_extents (PangoFont      *font,
 					       logical_rect);
 }
 
-static FT_Face
+static gpointer
 pango_cairo_fc_font_lock_face (PangoFcFont *font)
 {
   PangoCairoFcFont *cffont = (PangoCairoFcFont *) (font);
